import React from 'react'
import PropTypes from 'prop-types'
import { Title, Subtitle, Fields, Field } from '@foundation/core'
import { Card } from '@foundation/cards'
import { consolidate, flatten } from '../utils/index.js'


const DefaultLinkTooltip = ({
    link,
    position,
    definitions
}) => {
    const { source, target } = link
    const getName = obj => obj.attributes.name.full || obj.attributes.name

    return (
        <Card
            width={243}
            height={160}
            style={{
                position: 'absolute',
                marginLeft: -121.5,
                marginTop: -180,
                padding: 10,
                top: position.y,
                left: position.x,
                boxShadow: '0 0 25px #dedede',
                background: 'white'
            }}
        >
            <Title style={{ lineHeight: 1.2, marginBottom: -8 }}>
                {getName(source)} - {getName(target)}
            </Title>
            <Fields>
                {consolidate(flatten(link.match)).map(([field, value], i) => {
                    const parts = field.split('.')

                    return (
                        <Field
                            key={i}
                            title={field}
                            label={parts[parts.length - 1]}
                            style={{
                                textTransform: 'capitalize'
                            }}
                        >
                            {value}
                        </Field>
                    )
                })}
            </Fields>
        </Card>
    )
}

DefaultLinkTooltip.propTypes = {
    link: PropTypes.object,
    position: PropTypes.object,
    definitions: PropTypes.array
}

export default DefaultLinkTooltip