import React, { useRef } from 'react'
import PropTypes from 'prop-types'
import { useMeasure } from 'react-use'
import Graph from './graph.jsx'
import Tooltip from './tooltip.jsx'
import Zoomable from './zoomable.jsx'
import Draggable from './draggable.jsx'
import Selectable from './selectable.jsx'
import Nodes from './nodes.jsx'
import Links from './links.jsx'
import DefaultTooltip from './default-tooltip.jsx'
import { renderIcon as renderIconDefault } from '../utils/images.js'


const FoundationGraph = ({
    definitions = [],
    results,
    links,
    style,
    renderIcon = renderIconDefault,
    ...props
}) => {
    const [ref, {width, height}] = useMeasure()
    const tooltipContainer = useRef(null)
    const tipRef = useRef(null)
    const renderIconWrapped = (...args) => renderIcon(definitions, ...args)

    return (
        <div
            ref={ref}
            style={{
                position: 'absolute',
                top: 0,
                left: 0,
                width: '100%',
                height: '100%',
                ...style
            }}
        >
            <Graph
                width={width}
                height={height}
                nodes={results.map(obj => ({ ...obj }))}
                links={links.map(obj => ({ ...obj }))}
                {...props}
            >
                <Zoomable />
                <Draggable />
                <Selectable />
                <Links
                    onMouseMove={(link, position) => {
                        const { tooltip, setTooltip } = tipRef.current

                        if (!tooltip || tooltip.link) {
                            setTooltip(link && { link, position })
                        }
                    }}
                />
                <Nodes
                    image={renderIcon && renderIconWrapped}
                    onMouseMove={(node, position) => {
                        const { tooltip, setTooltip } = tipRef.current
                        
                        if (!node && tooltip && tooltip.link) return
                        else setTooltip(node && { node, position })
                    }}
                />
                <Tooltip
                    ref={tipRef}
                    container={tooltipContainer}
                    component={DefaultTooltip}
                    definitions={definitions}
                />
            </Graph>
            <aside ref={tooltipContainer} />
        </div>
    )
}

FoundationGraph.propTypes = {
    definitions: PropTypes.array,
    results: PropTypes.array,
    links: PropTypes.array,
    style: PropTypes.object,
    renderIcon: PropTypes.func
}

export default FoundationGraph