import React, { forwardRef, useState, useEffect, useImperativeHandle } from 'react'
import { render } from 'react-dom'
import PropTypes from 'prop-types'


const Tooltip = forwardRef(({
    component: Component,
    container,
    ...rest
}, ref) => {
    const [props, setProps] = useState(null)

    useImperativeHandle(ref, () => ({
        setTooltip: setProps,
        tooltip: props
    }))

    useEffect(() => {
        render(
            props ? <Component {...rest} {...props} /> : null,
            container.current
        )
    }, [ props ])

    return null
})

Tooltip.propTypes = {
    component: PropTypes.func.isRequired,
    container: PropTypes.object.isRequired
}

export default Tooltip